"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseUndiciResponseHeaders = exports.parseUndiciRequsetHeaders = exports.getClientUndiciResponseDetail = exports.getClientUndiciRequestDetail = void 0;
var stream_1 = require("stream");
var utils_1 = require("../utils");
function getClientUndiciRequestDetail(request, options) {
    if (options === void 0) { options = {}; }
    return new Promise(function (resolve, reject) {
        var headers = parseUndiciRequsetHeaders(request.headers);
        var req = {
            method: request.method,
            url: request.origin + request.path,
            time: Date.now(),
            headers: headers,
            contentType: (0, utils_1.hasBody)(request.method) ? ((0, utils_1.contentType)(headers) || undefined) : undefined,
        };
        if (options.body !== true || !req.contentType) {
            resolve(req);
            return;
        }
        var input = new stream_1.PassThrough();
        var decoders = (0, utils_1.createDecoders)(headers);
        var onError = request.onError;
        request.onError = function (error) {
            reject(error);
            input.end();
            return onError.call(this, error);
        };
        var onBodySent = request.onBodySent;
        request.onBodySent = function (chunk) {
            input.push(chunk);
            return onBodySent.call(this, chunk);
        };
        var onRequestSent = request.onRequestSent;
        request.onRequestSent = function () {
            input.end();
            return onRequestSent.call(this);
        };
        (0, utils_1.pipeline)(input, decoders)
            .then(function (buffer) {
            req.contentText = buffer === null || buffer === void 0 ? void 0 : buffer.toString('utf-8');
            resolve(req);
        })
            .catch(function (error) {
            reject(error);
        });
    });
}
exports.getClientUndiciRequestDetail = getClientUndiciRequestDetail;
function getClientUndiciResponseDetail(request, options) {
    if (options === void 0) { options = {}; }
    return new Promise(function (resolve, reject) {
        var res = {
            time: 0,
            status: 0,
            cost: 0,
            headers: [],
            contentType: undefined,
        };
        var input = null;
        var decoders = [];
        var onHeaders = request.onHeaders;
        request.onHeaders = function (status, headersList, resume) {
            var headers = parseUndiciResponseHeaders(headersList);
            res.status = status;
            res.headers = headers;
            res.time = Date.now();
            res.contentType = (0, utils_1.contentType)(headers) || undefined;
            if (options.body !== true || !res.contentType) {
                resolve(res);
            }
            else {
                input = new stream_1.PassThrough();
                decoders = (0, utils_1.createDecoders)(headers);
                (0, utils_1.pipeline)(input, decoders)
                    .then(function (buffer) {
                    res.contentText = buffer === null || buffer === void 0 ? void 0 : buffer.toString('utf-8');
                    resolve(res);
                })
                    .catch(function (error) {
                    reject(error);
                });
            }
            return onHeaders.call(this, status, headersList, resume);
        };
        var onData = request.onData;
        request.onData = function (chunk) {
            if (input) {
                input.push(chunk);
            }
            return onData.call(this, chunk);
        };
        var onComplete = request.onComplete;
        request.onComplete = function (trailers) {
            if (input) {
                input.end();
            }
            return onComplete.call(this, trailers);
        };
    });
}
exports.getClientUndiciResponseDetail = getClientUndiciResponseDetail;
function parseUndiciRequsetHeaders(headerString) {
    var result = [];
    var segments = headerString.trim().split(/[\n\r\t\f\v]+/);
    for (var _i = 0, segments_1 = segments; _i < segments_1.length; _i++) {
        var segment = segments_1[_i];
        var _a = segment.split(/\s*\:\s*/), _name = _a[0], _value = _a[1];
        var name_1 = _name.toLowerCase();
        var value = _value !== null && _value !== void 0 ? _value : '';
        if (!name_1) {
            continue;
        }
        result.push([name_1, value]);
    }
    return result;
}
exports.parseUndiciRequsetHeaders = parseUndiciRequsetHeaders;
function parseUndiciResponseHeaders(headersList) {
    var result = [];
    if (headersList) {
        for (var n = 0; n < headersList.length; n += 2) {
            var name_2 = headersList[n + 0].toString('latin1').toLowerCase();
            var value = headersList[n + 1].toString('latin1');
            result.push([name_2, value]);
        }
    }
    return result;
}
exports.parseUndiciResponseHeaders = parseUndiciResponseHeaders;
//# sourceMappingURL=undici.js.map