"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var http_1 = __importDefault(require("http"));
var https_1 = __importDefault(require("https"));
var utils_1 = require("../utils");
var http_2 = require("./http");
var _a = (0, utils_1.createListener)(), serverHttpInspector = _a[0], emit = _a[1];
var inspectedServerMethod = function (func) {
    var rest = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        rest[_i - 1] = arguments[_i];
    }
    var originHandler = typeof rest[rest.length - 1] === 'function' ? rest[rest.length - 1] : null;
    var handler = function (request, response) {
        var _this = this;
        (0, utils_1.startSequence)(function (sequence) {
            var _a;
            (0, utils_1.addSequence)(sequence);
            if (serverHttpInspector.hasListeners) {
                var callable_1 = true;
                var requestDetailBody_1 = false;
                var requestDetailResolvers_2 = [];
                var responseDetailBody_1 = false;
                var responseDetailResolvers_2 = [];
                emit({
                    url: request.url,
                    method: request.method,
                    time: Date.now(),
                    socket: {
                        remoteAddress: (_a = request.socket.remoteAddress) !== null && _a !== void 0 ? _a : '',
                        encrypted: request.socket.encrypted === true,
                    },
                    httpVersion: request.httpVersion,
                    httpVersionMajor: request.httpVersionMajor,
                    httpVersionMinor: request.httpVersionMinor,
                    requestHeaders: (0, http_2.parseHeaders)(request.headers),
                    getRequestDetail: function (options) {
                        if (!callable_1) {
                            return Promise.reject('event.getRequestDetail needs be called imminently');
                        }
                        if (requestDetailBody_1 === false && (options === null || options === void 0 ? void 0 : options.body) === true) {
                            requestDetailBody_1 = true;
                        }
                        return new Promise(function (resolve) { return requestDetailResolvers_2.push(resolve); });
                    },
                    getResponseDetail: function (options) {
                        if (!callable_1) {
                            return Promise.reject('event.getResponseDetail needs be called imminently');
                        }
                        if (responseDetailBody_1 === false && (options === null || options === void 0 ? void 0 : options.body) === true) {
                            responseDetailBody_1 = true;
                        }
                        return new Promise(function (resolve) { return responseDetailResolvers_2.push(resolve); });
                    },
                    sequence: sequence,
                });
                callable_1 = false;
                if (requestDetailResolvers_2.length) {
                    var promise = (0, http_2.getServerRequestDetail)(request, { body: requestDetailBody_1 }).catch(function (error) { return ({ error: (0, utils_1.errorToString)(error) }); });
                    for (var _i = 0, requestDetailResolvers_1 = requestDetailResolvers_2; _i < requestDetailResolvers_1.length; _i++) {
                        var resolver = requestDetailResolvers_1[_i];
                        resolver(promise);
                    }
                    requestDetailResolvers_2.length = 0;
                }
                if (responseDetailResolvers_2.length) {
                    var promise = (0, http_2.getServerResponseDetail)(response, { body: responseDetailBody_1 }).catch(function (error) { return ({ error: (0, utils_1.errorToString)(error) }); });
                    for (var _b = 0, responseDetailResolvers_1 = responseDetailResolvers_2; _b < responseDetailResolvers_1.length; _b++) {
                        var resolver = responseDetailResolvers_1[_b];
                        resolver(promise);
                    }
                    responseDetailResolvers_2.length = 0;
                }
            }
            if (originHandler) {
                originHandler.call(_this, request, response);
            }
        });
    };
    if (originHandler) {
        rest[rest.length - 1] = handler;
    }
    else {
        rest.push(handler);
    }
    return func.apply(this, rest);
};
http_1.default.createServer = inspectedServerMethod.bind(http_1.default, http_1.default.createServer);
https_1.default.createServer = inspectedServerMethod.bind(https_1.default, https_1.default.createServer);
exports.default = serverHttpInspector;
//# sourceMappingURL=index.js.map