"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.errorToString = exports.addSequence = exports.startSequence = exports.uuid = exports.createListener = exports.pipeline = exports.createDecoders = exports.contentType = exports.hasBody = void 0;
var type_is_1 = __importDefault(require("type-is"));
var async_hooks_1 = require("async_hooks");
var destroy_1 = __importDefault(require("destroy"));
var zlib_1 = __importDefault(require("zlib"));
function getHeader(name, headers) {
    name = name.toLowerCase();
    for (var _i = 0, headers_1 = headers; _i < headers_1.length; _i++) {
        var header = headers_1[_i];
        if (header[0] === name) {
            return header[1];
        }
    }
    return null;
}
function hasBody(method) {
    return ['POST', 'PUT', 'PATCH', 'DELETE'].includes(method.toUpperCase());
}
exports.hasBody = hasBody;
function contentType(headers) {
    var _a;
    var contentType = (_a = getHeader('content-type', headers)) !== null && _a !== void 0 ? _a : '';
    return type_is_1.default.is(contentType, ['urlencoded', 'json', 'text', 'html']);
}
exports.contentType = contentType;
var zlibOptions = {
    flush: zlib_1.default.constants.Z_SYNC_FLUSH,
    finishFlush: zlib_1.default.constants.Z_SYNC_FLUSH
};
var brotliOptions = {
    flush: zlib_1.default.constants.BROTLI_OPERATION_FLUSH,
    finishFlush: zlib_1.default.constants.BROTLI_OPERATION_FLUSH
};
function createDecoders(headers) {
    var _a;
    var decoders = [];
    var encodings = ((_a = getHeader('content-encoding', headers)) !== null && _a !== void 0 ? _a : '')
        .split(',')
        .map(function (val) { return val.trim().toLowerCase(); })
        .filter(function (val) { return val.length; });
    for (var _i = 0, encodings_1 = encodings; _i < encodings_1.length; _i++) {
        var encoding = encodings_1[_i];
        switch (encoding) {
            case 'gzip':
            case 'x-gzip':
            case 'compress':
            case 'x-compress':
            case 'deflate':
                decoders.push(zlib_1.default.createUnzip(zlibOptions));
                break;
            case 'br':
                decoders.push(zlib_1.default.createBrotliDecompress(brotliOptions));
                break;
        }
    }
    return decoders;
}
exports.createDecoders = createDecoders;
function pipeline(input, transforms) {
    return new Promise(function (resolve, reject) {
        var _a;
        for (var i = 0; i < transforms.length; i++) {
            ((_a = transforms[i - 1]) !== null && _a !== void 0 ? _a : input).pipe(transforms[i]);
        }
        var output = transforms.length ? transforms[transforms.length - 1] : input;
        var chunks = [];
        output.on('data', function (chunk) {
            chunks.push(chunk);
        });
        output.on('end', function () {
            (0, destroy_1.default)(input);
            for (var _i = 0, transforms_1 = transforms; _i < transforms_1.length; _i++) {
                var decoder = transforms_1[_i];
                (0, destroy_1.default)(decoder);
            }
            resolve(Buffer.concat(chunks));
        });
        output.on('error', function (error) {
            (0, destroy_1.default)(input);
            for (var _i = 0, transforms_2 = transforms; _i < transforms_2.length; _i++) {
                var decoder = transforms_2[_i];
                (0, destroy_1.default)(decoder);
            }
            reject(error);
        });
    });
}
exports.pipeline = pipeline;
function createListener() {
    var listeners = [];
    var listener = {
        get hasListeners() {
            return listeners.length > 0;
        },
        addListener: function (fn) {
            listeners.push(fn);
            return function () {
                var index = listeners.indexOf(fn);
                if (index >= 0) {
                    listeners.splice(index, 1);
                }
            };
        },
        removeAllListeners: function () {
            listeners.length = 0;
        }
    };
    var emit = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        for (var _a = 0, listeners_1 = listeners; _a < listeners_1.length; _a++) {
            var listener_1 = listeners_1[_a];
            listener_1.apply(void 0, args);
        }
    };
    return [listener, emit];
}
exports.createListener = createListener;
var rand = Math.random;
var hexmap = [0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66];
var arr = new Array(36);
function uuid() {
    setChars12(arr, rand(), 0);
    setChars12(arr, rand(), 12);
    setChars12(arr, rand(), 24);
    arr[8] = 0x2d; // -
    arr[13] = 0x2d; // -
    arr[14] = 0x34; // 4
    arr[18] = 0x2d; // -
    arr[19] = hexmap[8 + (arr[19] & 3)]; // [89ab]
    arr[23] = 0x2d; // -
    return String.fromCharCode.apply(String, arr);
}
exports.uuid = uuid;
function setChars12(buf, n, pos) {
    for (var i = 0; i < 12; i += 3) {
        n *= 0x1000;
        buf[pos + i + 0] = hexmap[(n) & 0xF];
        buf[pos + i + 1] = hexmap[(n >>> 4) & 0xF];
        buf[pos + i + 2] = hexmap[(n >>> 8) & 0xF];
    }
}
var sequenceContext = new async_hooks_1.AsyncLocalStorage();
function startSequence(callback) {
    var store = { id: uuid(), index: -1 };
    sequenceContext.run(store, function () { return callback(store); });
}
exports.startSequence = startSequence;
function addSequence(sequenceData) {
    var store = sequenceData !== null && sequenceData !== void 0 ? sequenceData : sequenceContext.getStore();
    if (store) {
        store.index++;
        return store;
    }
    return null;
}
exports.addSequence = addSequence;
function errorToString(error) {
    if (error instanceof Error) {
        return error.message;
    }
    return String(error);
}
exports.errorToString = errorToString;
//# sourceMappingURL=utils.js.map